@@ -32,7 +32,7 @@ from gitlab.exceptions import *  # noqa
 from gitlab.objects import *  # noqa
 
 __title__ = 'python-gitlab'
-__version__ = '0.11.1'
+__version__ = '0.12.1'
 __author__ = 'Gauvain Pocentek'
 __email__ = 'gauvain@pocentek.net'
 __license__ = 'LGPL3'
@@ -122,6 +122,7 @@ class Gitlab(object):
         #: Whether SSL certificates should be validated
         self.ssl_verify = ssl_verify
 
+        self.settings = ApplicationSettingsManager(self)
         self.user_keys = UserKeyManager(self)
         self.users = UserManager(self)
         self.group_members = GroupMemberManager(self)
@@ -264,9 +265,9 @@ class Gitlab(object):
                                 headers=headers,
                                 verify=self.ssl_verify,
                                 timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
     def _raw_post(self, path, data=None, content_type=None, **kwargs):
         url = '%s%s' % (self._url, path)
@@ -276,9 +277,9 @@ class Gitlab(object):
                                  headers=headers,
                                  verify=self.ssl_verify,
                                  timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
     def _raw_put(self, path, data=None, content_type=None, **kwargs):
         url = '%s%s' % (self._url, path)
@@ -289,9 +290,9 @@ class Gitlab(object):
                                 headers=headers,
                                 verify=self.ssl_verify,
                                 timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
     def _raw_delete(self, path, content_type=None, **kwargs):
         url = '%s%s' % (self._url, path)
@@ -303,9 +304,9 @@ class Gitlab(object):
                                    headers=headers,
                                    verify=self.ssl_verify,
                                    timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
     def list(self, obj_class, **kwargs):
         """Request the listing of GitLab resources.
@@ -343,9 +344,9 @@ class Gitlab(object):
             r = requests.get(url, params=params, headers=headers,
                              verify=self.ssl_verify,
                              timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
         raise_error_from_response(r, GitlabListError)
 
@@ -413,9 +414,9 @@ class Gitlab(object):
         try:
             r = requests.get(url, params=params, headers=headers,
                              verify=self.ssl_verify, timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
         raise_error_from_response(r, GitlabGetError)
         return r.json()
@@ -454,7 +455,7 @@ class Gitlab(object):
             raise GitlabDeleteError('Missing attribute(s): %s' %
                                     ", ".join(missing))
 
-        obj_id = params[obj.idAttr]
+        obj_id = params[obj.idAttr] if obj._id_in_delete_url else None
         url = self._construct_url(id_=obj_id, obj=obj, parameters=params)
         headers = self._create_headers()
 
@@ -469,9 +470,9 @@ class Gitlab(object):
                                 headers=headers,
                                 verify=self.ssl_verify,
                                 timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
         raise_error_from_response(r, GitlabDeleteError)
         return True
@@ -516,9 +517,9 @@ class Gitlab(object):
                               headers=headers,
                               verify=self.ssl_verify,
                               timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
         raise_error_from_response(r, GitlabCreateError, 201)
         return r.json()
@@ -551,20 +552,21 @@ class Gitlab(object):
         if missing:
             raise GitlabUpdateError('Missing attribute(s): %s' %
                                     ", ".join(missing))
-        url = self._construct_url(id_=obj.id, obj=obj, parameters=params)
+        obj_id = params[obj.idAttr] if obj._id_in_update_url else None
+        url = self._construct_url(id_=obj_id, obj=obj, parameters=params)
         headers = self._create_headers(content_type="application/json")
 
         # build data that can really be sent to server
-        data = obj._data_for_gitlab(extra_parameters=kwargs)
+        data = obj._data_for_gitlab(extra_parameters=kwargs, update=True)
 
         try:
             r = requests.put(url, data=data,
                              headers=headers,
                              verify=self.ssl_verify,
                              timeout=self.timeout)
-        except Exception:
+        except Exception as e:
             raise GitlabConnectionError(
-                "Can't connect to GitLab server (%s)" % self._url)
+                "Can't connect to GitLab server (%s)" % e)
 
         raise_error_from_response(r, GitlabUpdateError)
         return r.json()
