@@ -73,6 +73,7 @@ def _populate_sub_parser_by_class(cls, sub_parser):
         [sub_parser_action.add_argument("--%s" % x.replace('_', '-'),
                                         required=True)
          for x in cls.requiredUrlAttrs]
+        sub_parser_action.add_argument("--sudo", required=False)
 
         if action_name == LIST:
             [sub_parser_action.add_argument("--%s" % x.replace('_', '-'),
@@ -89,7 +90,7 @@ def _populate_sub_parser_by_class(cls, sub_parser):
                                                    required=True)
                 [sub_parser_action.add_argument("--%s" % x.replace('_', '-'),
                                                 required=True)
-                 for x in cls.requiredGetAttrs]
+                 for x in cls.requiredGetAttrs if x != cls.idAttr]
 
         elif action_name == CREATE:
             [sub_parser_action.add_argument("--%s" % x.replace('_', '-'),
@@ -105,14 +106,14 @@ def _populate_sub_parser_by_class(cls, sub_parser):
                                            required=True)
 
             attrs = (cls.requiredUpdateAttrs
-                     if cls.requiredUpdateAttrs is not None
+                     if (cls.requiredUpdateAttrs or cls.optionalUpdateAttrs)
                      else cls.requiredCreateAttrs)
             [sub_parser_action.add_argument("--%s" % x.replace('_', '-'),
                                             required=True)
-             for x in attrs]
+             for x in attrs if x != cls.idAttr]
 
             attrs = (cls.optionalUpdateAttrs
-                     if cls.optionalUpdateAttrs is not None
+                     if (cls.requiredUpdateAttrs or cls.optionalUpdateAttrs)
                      else cls.optionalCreateAttrs)
             [sub_parser_action.add_argument("--%s" % x.replace('_', '-'),
                                             required=False)
@@ -239,8 +240,14 @@ def do_project_owned(gl, what, args):
 
 
 def main():
+    if "--version" in sys.argv:
+        print(gitlab.__version__)
+        exit(0)
+
     parser = argparse.ArgumentParser(
         description="GitLab API Command Line Interface")
+    parser.add_argument("--version", help="Display the version.",
+                        action="store_true")
     parser.add_argument("-v", "--verbose", "--fancy",
                         help="Verbose mode",
                         action="store_true")
@@ -330,6 +337,7 @@ def main():
 
         for o in l:
             o.display(verbose)
+            print("")
 
     elif action == OWNED:
         if cls != gitlab.Project:
@@ -337,6 +345,7 @@ def main():
 
         for o in do_project_owned(gl, what, args):
             o.display(verbose)
+            print("")
 
     elif action == ALL:
         if cls != gitlab.Project:
@@ -344,5 +353,6 @@ def main():
 
         for o in do_project_all(gl, what, args):
             o.display(verbose)
+            print("")
 
     sys.exit(0)
