@@ -18,6 +18,7 @@
 from __future__ import print_function
 from __future__ import division
 from __future__ import absolute_import
+import base64
 import copy
 import itertools
 import json
@@ -26,14 +27,17 @@ import warnings
 
 import six
 
+import gitlab
 from gitlab.exceptions import *  # noqa
 
 
 class jsonEncoder(json.JSONEncoder):
     def default(self, obj):
         if isinstance(obj, GitlabObject):
-            return obj.__dict__
-        elif isinstance(obj, Gitlab):
+            return {k: v for k, v in six.iteritems(obj.__dict__)
+                    if (not isinstance(v, BaseManager)
+                        and not k[0] == '_')}
+        elif isinstance(obj, gitlab.Gitlab):
             return {'url': obj._url}
         return json.JSONEncoder.default(self, obj)
 
@@ -173,11 +177,13 @@ class GitlabObject(object):
     # Some objects (e.g. merge requests) have different urls for singular and
     # plural
     _urlPlural = None
+    _id_in_delete_url = True
+    _id_in_update_url = True
     _returnClass = None
     _constructorTypes = None
+
     #: Whether _get_list_or_object should return list or object when id is None
     getListWhenNoId = True
-
     #: Tells if GitLab-api allows retrieving single objects.
     canGet = True
     #: Tells if GitLab-api allows listing of objects.
@@ -201,9 +207,9 @@ class GitlabObject(object):
     #: Attributes that are optional when creating a new object.
     optionalCreateAttrs = []
     #: Attributes that are required when updating an object.
-    requiredUpdateAttrs = None
+    requiredUpdateAttrs = []
     #: Attributes that are optional when updating an object.
-    optionalUpdateAttrs = None
+    optionalUpdateAttrs = []
     #: Whether the object ID is required in the GET url.
     getRequiresId = True
     #: List of managers to create.
@@ -213,10 +219,16 @@ class GitlabObject(object):
     #: Attribute to use as ID when displaying the object.
     shortPrintAttr = None
 
-    def _data_for_gitlab(self, extra_parameters={}):
+    def _data_for_gitlab(self, extra_parameters={}, update=False):
         data = {}
-        for attribute in itertools.chain(self.requiredCreateAttrs,
-                                         self.optionalCreateAttrs):
+        if update and (self.requiredUpdateAttrs or self.optionalUpdateAttrs):
+            attributes = itertools.chain(self.requiredUpdateAttrs,
+                                         self.optionalUpdateAttrs)
+        else:
+            attributes = itertools.chain(self.requiredCreateAttrs,
+                                         self.optionalCreateAttrs)
+        attributes = list(attributes) + ['sudo', 'page', 'per_page']
+        for attribute in attributes:
             if hasattr(self, attribute):
                 data[attribute] = getattr(self, attribute)
 
@@ -474,7 +486,7 @@ class GitlabObject(object):
         Returns:
             str: The json string.
         """
-        return json.dumps(self.__dict__, cls=jsonEncoder)
+        return json.dumps(self, cls=jsonEncoder)
 
 
 class UserKey(GitlabObject):
@@ -492,15 +504,18 @@ class UserKeyManager(BaseManager):
 class User(GitlabObject):
     _url = '/users'
     shortPrintAttr = 'username'
-    # FIXME: password is required for create but not for update
-    requiredCreateAttrs = ['email', 'username', 'name']
-    optionalCreateAttrs = ['password', 'skype', 'linkedin', 'twitter',
-                           'projects_limit', 'extern_uid', 'provider',
-                           'bio', 'admin', 'can_create_group', 'website_url',
+    requiredCreateAttrs = ['email', 'username', 'name', 'password']
+    optionalCreateAttrs = ['skype', 'linkedin', 'twitter', 'projects_limit',
+                           'extern_uid', 'provider', 'bio', 'admin',
+                           'can_create_group', 'website_url', 'confirm']
+    requiredUpdateAttrs = ['email', 'username', 'name']
+    optionalUpdateAttrs = ['password', 'skype', 'linkedin', 'twitter',
+                           'projects_limit', 'extern_uid', 'provider', 'bio',
+                           'admin', 'can_create_group', 'website_url',
                            'confirm']
     managers = [('keys', UserKeyManager, [('user_id', 'id')])]
 
-    def _data_for_gitlab(self, extra_parameters={}):
+    def _data_for_gitlab(self, extra_parameters={}, update=False):
         if hasattr(self, 'confirm'):
             self.confirm = str(self.confirm).lower()
         return super(User, self)._data_for_gitlab(extra_parameters)
@@ -543,6 +558,28 @@ class CurrentUser(GitlabObject):
         return CurrentUserKey._get_list_or_object(self.gitlab, id, **kwargs)
 
 
+class ApplicationSettings(GitlabObject):
+    _url = '/application/settings'
+    _id_in_update_url = False
+    optionalUpdateAttrs = ['after_sign_out_path', 'default_branch_protection',
+                           'default_project_visibility',
+                           'default_projects_limit',
+                           'default_snippet_visibility', 'gravatar_enabled',
+                           'home_page_url', 'restricted_signup_domains',
+                           'restricted_visibility_levels',
+                           'session_expire_delay', 'sign_in_text',
+                           'signin_enabled', 'signup_enabled',
+                           'twitter_sharing_enabled',
+                           'user_oauth_applications']
+    canList = False
+    canCreate = False
+    canDelete = False
+
+
+class ApplicationSettingsManager(BaseManager):
+    obj_cls = ApplicationSettings
+
+
 class GroupMember(GitlabObject):
     _url = '/groups/%(group_id)s/members'
     canGet = 'from_list'
@@ -582,6 +619,16 @@ class Group(GitlabObject):
                                                **kwargs)
 
     def transfer_project(self, id, **kwargs):
+        """Transfers a project to this new groups.
+
+        Attrs:
+            id (int): ID of the project to transfer.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabTransferProjectError: If the server fails to perform the
+                request.
+        """
         url = '/groups/%d/projects/%d' % (self.id, id)
         r = self.gitlab._raw_post(url, None, **kwargs)
         raise_error_from_response(r, GitlabTransferProjectError, 201)
@@ -633,9 +680,9 @@ class ProjectBranch(GitlabObject):
     canUpdate = False
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['branch_name', 'ref']
-    _constructorTypes = {'commit': 'ProjectCommit'}
 
     def protect(self, protect=True, **kwargs):
+        """Protects the project."""
         url = self._url % {'project_id': self.project_id}
         action = 'protect' if protect else 'unprotect'
         url = "%s/%s/%s" % (url, self.name, action)
@@ -648,6 +695,7 @@ class ProjectBranch(GitlabObject):
             del self.protected
 
     def unprotect(self, **kwargs):
+        """Unprotects the project."""
         self.protect(False, **kwargs)
 
 
@@ -655,6 +703,32 @@ class ProjectBranchManager(BaseManager):
     obj_cls = ProjectBranch
 
 
+class ProjectBuild(GitlabObject):
+    _url = '/projects/%(project_id)s/builds'
+    _constructorTypes = {'user': 'User',
+                         'commit': 'ProjectCommit'}
+    requiredUrlAttrs = ['project_id']
+    canDelete = False
+    canUpdate = False
+    canCreate = False
+
+    def cancel(self):
+        """Cancel the build."""
+        url = '/projects/%s/builds/%s/cancel' % (self.project_id, self.id)
+        r = self.gitlab._raw_post(url)
+        raise_error_from_response(r, GitlabBuildCancelError, 201)
+
+    def retry(self):
+        """Retry the build."""
+        url = '/projects/%s/builds/%s/retry' % (self.project_id, self.id)
+        r = self.gitlab._raw_post(url)
+        raise_error_from_response(r, GitlabBuildRetryError, 201)
+
+
+class ProjectBuildManager(BaseManager):
+    obj_cls = ProjectBuild
+
+
 class ProjectCommit(GitlabObject):
     _url = '/projects/%(project_id)s/repository/commits'
     canDelete = False
@@ -664,6 +738,7 @@ class ProjectCommit(GitlabObject):
     shortPrintAttr = 'title'
 
     def diff(self, **kwargs):
+        """Generate the commit diff."""
         url = ('/projects/%(project_id)s/repository/commits/%(commit_id)s/diff'
                % {'project_id': self.project_id, 'commit_id': self.id})
         r = self.gitlab._raw_get(url, **kwargs)
@@ -672,6 +747,18 @@ class ProjectCommit(GitlabObject):
         return r.json()
 
     def blob(self, filepath, **kwargs):
+        """Generate the content of a file for this commit.
+
+        Args:
+            filepath (str): Path of the file to request.
+
+        Returns:
+            str: The content of the file
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabGetError: If the server fails to perform the request.
+        """
         url = ('/projects/%(project_id)s/repository/blobs/%(commit_id)s' %
                {'project_id': self.project_id, 'commit_id': self.id})
         url += '?filepath=%s' % filepath
@@ -680,6 +767,29 @@ class ProjectCommit(GitlabObject):
 
         return r.content
 
+    def builds(self, **kwargs):
+        """List the build for this commit.
+
+        Returns:
+            list(ProjectBuild): A list of builds.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabListError: If the server fails to perform the request.
+        """
+        url = '/projects/%s/repository/commits/%s/builds' % (self.project_id,
+                                                             self.id)
+        r = self.gitlab._raw_get(url, **kwargs)
+        raise_error_from_response(r, GitlabListError)
+
+        l = []
+        for j in r.json():
+            o = ProjectBuild(self, j)
+            o._from_api = True
+            l.append(o)
+
+        return l
+
 
 class ProjectCommitManager(BaseManager):
     obj_cls = ProjectCommit
@@ -692,9 +802,6 @@ class ProjectCommitStatus(GitlabObject):
     requiredUrlAttrs = ['project_id', 'commit_id']
     requiredCreateAttrs = ['state']
     optionalCreateAttrs = ['description', 'name', 'ref', 'target_url']
-    requiredGetAttrs = []
-    requiredUpdateAttrs = []
-    requiredDeleteAttrs = []
 
 
 class ProjectCommitStatusManager(BaseManager):
@@ -779,7 +886,7 @@ class ProjectIssue(GitlabObject):
     managers = [('notes', ProjectIssueNoteManager,
                  [('project_id', 'project_id'), ('issue_id', 'id')])]
 
-    def _data_for_gitlab(self, extra_parameters={}):
+    def _data_for_gitlab(self, extra_parameters={}, update=False):
         # Gitlab-api returns labels in a json list and takes them in a
         # comma separated list.
         if hasattr(self, "labels"):
@@ -827,17 +934,55 @@ class ProjectNoteManager(BaseManager):
     obj_cls = ProjectNote
 
 
+class ProjectTagRelease(GitlabObject):
+    _url = '/projects/%(project_id)s/repository/tags/%(tag_name)/release'
+    canDelete = False
+    canList = False
+    requiredUrlAttrs = ['project_id', 'tag_name']
+    requiredCreateAttrs = ['description']
+    shortPrintAttr = 'description'
+
+
+class ProjectTagReleaseManager(BaseManager):
+    obj_cls = ProjectTagRelease
+
+
 class ProjectTag(GitlabObject):
     _url = '/projects/%(project_id)s/repository/tags'
+    _constructorTypes = {'release': 'ProjectTagRelease',
+                         'commit': 'ProjectCommit'}
     idAttr = 'name'
     canGet = 'from_list'
-    canDelete = False
     canUpdate = False
     requiredUrlAttrs = ['project_id']
     requiredCreateAttrs = ['tag_name', 'ref']
     optionalCreateAttrs = ['message']
     shortPrintAttr = 'name'
 
+    def set_release_description(self, description):
+        """Set the release notes on the tag.
+
+        If the release doesn't exist yet, it will be created. If it already
+        exists, its description will be updated.
+
+        Args:
+            description (str): Description of the release.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabCreateError: If the server fails to create the release.
+            GitlabUpdateError: If the server fails to update the release.
+        """
+        url = '/projects/%s/repository/tags/%s/release' % (self.project_id,
+                                                           self.name)
+        if self.release is None:
+            r = self.gitlab._raw_post(url, data={'description': description})
+            raise_error_from_response(r, GitlabCreateError, 201)
+        else:
+            r = self.gitlab._raw_put(url, data={'description': description})
+            raise_error_from_response(r, GitlabUpdateError, 200)
+        self.release = ProjectTagRelease(self, r.json())
+
 
 class ProjectTagManager(BaseManager):
     obj_cls = ProjectTag
@@ -886,6 +1031,19 @@ class ProjectMilestone(GitlabObject):
     optionalCreateAttrs = ['description', 'due_date', 'state_event']
     shortPrintAttr = 'title'
 
+    def issues(self):
+        url = "/projects/%s/milestones/%s/issues" % (self.project_id, self.id)
+        r = self.gitlab._raw_get(url)
+        raise_error_from_response(r, GitlabDeleteError)
+
+        l = []
+        for j in r.json():
+            o = ProjectIssue(self, j)
+            o._from_api = True
+            l.append(o)
+
+        return l
+
 
 class ProjectMilestoneManager(BaseManager):
     obj_cls = ProjectMilestone
@@ -893,6 +1051,9 @@ class ProjectMilestoneManager(BaseManager):
 
 class ProjectLabel(GitlabObject):
     _url = '/projects/%(project_id)s/labels'
+    _id_in_delete_url = False
+    _id_in_update_url = False
+    canGet = 'from_list'
     requiredUrlAttrs = ['project_id']
     idAttr = 'name'
     requiredDeleteAttrs = ['name']
@@ -919,6 +1080,14 @@ class ProjectFile(GitlabObject):
     shortPrintAttr = 'file_path'
     getRequiresId = False
 
+    def decode(self):
+        """Returns the decoded content of the file.
+
+        Returns:
+            (str): the decoded content.
+        """
+        return base64.b64decode(self.content)
+
 
 class ProjectFileManager(BaseManager):
     obj_cls = ProjectFile
@@ -968,18 +1137,45 @@ class ProjectSnippetManager(BaseManager):
     obj_cls = ProjectSnippet
 
 
+class ProjectTrigger(GitlabObject):
+    _url = '/projects/%(project_id)s/triggers'
+    canUpdate = False
+    idAttr = 'token'
+    requiredUrlAttrs = ['project_id']
+
+
+class ProjectTriggerManager(BaseManager):
+    obj_cls = ProjectTrigger
+
+
+class ProjectVariable(GitlabObject):
+    _url = '/projects/%(project_id)s/variables'
+    idAttr = 'key'
+    requiredUrlAttrs = ['project_id']
+    requiredCreateAttrs = ['key', 'value']
+
+
+class ProjectVariableManager(BaseManager):
+    obj_cls = ProjectVariable
+
+
 class Project(GitlabObject):
     _url = '/projects'
     _constructorTypes = {'owner': 'User', 'namespace': 'Group'}
     requiredCreateAttrs = ['name']
-    requiredUpdateAttrs = []
     optionalCreateAttrs = ['default_branch', 'issues_enabled', 'wall_enabled',
                            'merge_requests_enabled', 'wiki_enabled',
                            'snippets_enabled', 'public', 'visibility_level',
                            'namespace_id', 'description', 'path', 'import_url']
+    optionalUpdateAttrs = ['name', 'default_branch', 'issues_enabled',
+                           'wall_enabled', 'merge_requests_enabled',
+                           'wiki_enabled', 'snippets_enabled', 'public',
+                           'visibility_level', 'namespace_id', 'description',
+                           'path', 'import_url']
     shortPrintAttr = 'path'
     managers = [
         ('branches', ProjectBranchManager, [('project_id', 'id')]),
+        ('builds', ProjectBuildManager, [('project_id', 'id')]),
         ('commits', ProjectCommitManager, [('project_id', 'id')]),
         ('commitstatuses', ProjectCommitStatusManager, [('project_id', 'id')]),
         ('events', ProjectEventManager, [('project_id', 'id')]),
@@ -995,6 +1191,8 @@ class Project(GitlabObject):
         ('notes', ProjectNoteManager, [('project_id', 'id')]),
         ('snippets', ProjectSnippetManager, [('project_id', 'id')]),
         ('tags', ProjectTagManager, [('project_id', 'id')]),
+        ('triggers', ProjectTriggerManager, [('project_id', 'id')]),
+        ('variables', ProjectVariableManager, [('project_id', 'id')]),
     ]
 
     def Branch(self, id=None, **kwargs):
@@ -1097,20 +1295,64 @@ class Project(GitlabObject):
                                               **kwargs)
 
     def tree(self, path='', ref_name='', **kwargs):
-        url = "%s/%s/repository/tree" % (self._url, self.id)
-        url += '?path=%s&ref_name=%s' % (path, ref_name)
+        """Return a list of files in the repository.
+
+        Args:
+            path (str): Path of the top folder (/ by default)
+            ref_name (str): Reference to a commit or branch
+
+        Returns:
+            str: The json representation of the tree.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabGetError: If the server fails to perform the request.
+        """
+        url = "/projects/%s/repository/tree" % (self.id)
+        params = []
+        if path:
+            params.append("path=%s" % path)
+        if ref_name:
+            params.append("ref_name=%s" % ref_name)
+        if params:
+            url += '?' + "&".join(params)
         r = self.gitlab._raw_get(url, **kwargs)
         raise_error_from_response(r, GitlabGetError)
         return r.json()
 
     def blob(self, sha, filepath, **kwargs):
-        url = "%s/%s/repository/blobs/%s" % (self._url, self.id, sha)
+        """Return the content of a file for a commit.
+
+        Args:
+            sha (str): ID of the commit
+            filepath (str): Path of the file to return
+
+        Returns:
+            str: The file content
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabGetError: If the server fails to perform the request.
+        """
+        url = "/projects/%s/repository/blobs/%s" % (self.id, sha)
         url += '?filepath=%s' % (filepath)
         r = self.gitlab._raw_get(url, **kwargs)
         raise_error_from_response(r, GitlabGetError)
         return r.content
 
     def archive(self, sha=None, **kwargs):
+        """Return a tarball of the repository.
+
+        Args:
+            sha (str): ID of the commit (default branch by default).
+
+        Returns:
+            str: The binary data of the archive.
+
+        Raises:
+            GitlabConnectionError: If the server cannot be reached.
+            GitlabGetError: If the server fails to perform the request.
+        """
         url = '/projects/%s/repository/archive' % self.id
         if sha:
             url += '?sha=%s' % sha
